# Authors:	Cung Lian Sang, Bastian Steinhagen, Michael Adams
# Emails:	{csang, bsteinhagen, madams}@techfak.uni-bielefeld.de
# Cognitronics and Sensor Systems Group, CITEC, Bielefeld University

# Import pandas and numpy for data manipulation
import pandas as pd
import numpy as np
from time import time

# Import modules from Scikit-learn
from sklearn.svm import SVC                            # Import SVM model using guassian
from sklearn.model_selection import train_test_split   # Import train_test_split function
from sklearn import metrics                            # import metrics modules for accuracy calculation
from sklearn.metrics import plot_confusion_matrix
from sklearn.preprocessing import StandardScaler
from sklearn.pipeline import make_pipeline
from joblib import dump, load                      # For saving and loading the trained model


# import graphics tools
import matplotlib.pyplot as plt

# Create a pipeline object for our model SVM using rbf kernel
pipe_SVM = make_pipeline(StandardScaler(),
                         SVC(kernel='rbf',      # radial basic function kernel, i.e. gaussian kernel in matlab
                         cache_size=1000,       # Default is 200 MB. memory intensiveness vs. training time
                         verbose=True)
                         )
print("Data Pipeline (StandardScaler >> SVC) is created!\n")
print(pipe_SVM)

# Load the measured data in CSV format using pandas
#uwb_raw = "../uwb_raw_exp_data/unbalanced_uwb_FullDataSet.txt"
uwb_raw = "../uwb_raw_exp_data/downSample_uwb_FullDataSet.txt"
#uwb_raw = "../uwb_raw_exp_data/upsample_uwb_FullDataSet.txt"
data = pd.read_csv(uwb_raw)
#print(data.shape)
#print(data)
data.head()

# Extracted features in the evaluations
#X = data[['Dist', 'FP', 'FPAmp1', 'FPAmp2', 'FPAmp3', 'CIR', 'stdN', 'maxN', 'preamCnt', 'RSS_CIR', 'RSS_FP', 'powerDiff']]  # 12 features
#X = data[['Dist', 'FP', 'FPAmp1', 'FPAmp2', 'FPAmp3', 'CIR', 'preamCnt', 'RSS_CIR', 'RSS_FP', 'powerDiff']]  # 10 features
#X = data[['Dist', 'FPAmp1', 'FPAmp2', 'FPAmp3', 'CIR']]  # 5 Features
X = data[['Dist', 'FPAmp1', 'CIR']]  # Only 3 Features
#X = data[['Dist', 'FPAmp2', 'CIR']]  # Only 3 Features
#X = data[['Dist', 'FP', 'CIR']]  # Only 3 Features
#X = data[['FPAmp1', 'CIR']]  # Only 2 Features
#X = data[['Dist', 'FPAmp1']]  # Only 2 Features

y = data['Label']  # Labels
print("selected features: \n", X)
print("3 classes labels:\n", y)

# Split dataset into training and test set
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size = 0.3) # 70% training and 30% test

print("\nTraining in progress! This may take some times ... \n")
t0 = time()
# Train the whole pipeline
pipe_SVM.fit(X_train, y_train)
print("Training Time: %0.3fs" % (time() - t0))

# Predict the whole pipeline
t1 = time()
y_pred = pipe_SVM.predict(X_test)
print("Testing Time: %0.3fs" % (time() - t1))

# Access the classifier within the pipeline
clf_svm = pipe_SVM.named_steps.svc   # or pipe_MLP.steps[1]   # steps[0] is scaler in our case

#Model accuracy, how often is the classifier correct?
print("\nTest Accuracy in SVM:", metrics.accuracy_score(y_test, y_pred))

# Predict using the seen trained data
y_pred_seen = pipe_SVM.predict(X_train)
print("Training Accuracy in SVM:", metrics.accuracy_score(y_train, y_pred_seen))


# Plot the results and Evaluate
class_name = ['LOS', 'NLOS', 'MP']         # Create the class name to dply

# Plot Confusion Matrix (CM) with normalized value
cm_SVM = plot_confusion_matrix(pipe_SVM, X_test, y_test, normalize='true', display_labels=class_name)

print("\n Confusion matrix: \n", cm_SVM.confusion_matrix)

print(y_pred.shape)
print("\nresults save in the file name 'SVM_pred_true_pipeline.txt'\n")
result = np.column_stack((y_pred, y_test))       # combine numpy vectors as a column stack
np.savetxt("SVM_pred_true_pipeline.txt", result, delimiter=',', header="Predict_Test, True_TestLabel", comments="")
print(result[0:10])

# Save the results from training too for verification purposes
result_train = np.column_stack((y_pred_seen, y_train))
np.savetxt("SVM_pred_true_pipeline_trainingResult.txt", result_train, delimiter=',', header='Predict_Seen_train, True_TrainLabel')

# Save the trained model
dump(pipe_SVM, 'trained_models/trained_SVM_pipe.joblib')
